"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.distribution.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service for distribution module."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryParamsResponse]
    """Params queries params of the distribution module."""

    ValidatorOutstandingRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsResponse]
    """ValidatorOutstandingRewards queries rewards of a validator address."""

    ValidatorCommission: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionResponse]
    """ValidatorCommission queries accumulated commission for a validator."""

    ValidatorSlashes: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesResponse]
    """ValidatorSlashes queries slash events of a validator."""

    DelegationRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsResponse]
    """DelegationRewards queries the total rewards accrued by a delegation."""

    DelegationTotalRewards: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsResponse]
    """DelegationTotalRewards queries the total rewards accrued by a each
    validator.
    """

    DelegatorValidators: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsResponse]
    """DelegatorValidators queries the validators of a delegator."""

    DelegatorWithdrawAddress: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressResponse]
    """DelegatorWithdrawAddress queries withdraw address of a delegator."""

    CommunityPool: grpc.UnaryUnaryMultiCallable[
        cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolRequest,
        cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolResponse]
    """CommunityPool queries the community pool coins."""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service for distribution module."""
    @abc.abstractmethod
    def Params(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries params of the distribution module."""
        pass

    @abc.abstractmethod
    def ValidatorOutstandingRewards(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorOutstandingRewardsResponse:
        """ValidatorOutstandingRewards queries rewards of a validator address."""
        pass

    @abc.abstractmethod
    def ValidatorCommission(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorCommissionResponse:
        """ValidatorCommission queries accumulated commission for a validator."""
        pass

    @abc.abstractmethod
    def ValidatorSlashes(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryValidatorSlashesResponse:
        """ValidatorSlashes queries slash events of a validator."""
        pass

    @abc.abstractmethod
    def DelegationRewards(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegationRewardsResponse:
        """DelegationRewards queries the total rewards accrued by a delegation."""
        pass

    @abc.abstractmethod
    def DelegationTotalRewards(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegationTotalRewardsResponse:
        """DelegationTotalRewards queries the total rewards accrued by a each
        validator.
        """
        pass

    @abc.abstractmethod
    def DelegatorValidators(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegatorValidatorsResponse:
        """DelegatorValidators queries the validators of a delegator."""
        pass

    @abc.abstractmethod
    def DelegatorWithdrawAddress(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryDelegatorWithdrawAddressResponse:
        """DelegatorWithdrawAddress queries withdraw address of a delegator."""
        pass

    @abc.abstractmethod
    def CommunityPool(self,
        request: cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.distribution.v1beta1.query_pb2.QueryCommunityPoolResponse:
        """CommunityPool queries the community pool coins."""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgUnjail(google.protobuf.message.Message):
    """MsgUnjail defines the Msg/Unjail request type"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_ADDR_FIELD_NUMBER: builtins.int
    validator_addr: typing.Text
    def __init__(self,
        *,
        validator_addr: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["validator_addr",b"validator_addr"]) -> None: ...
global___MsgUnjail = MsgUnjail

class MsgUnjailResponse(google.protobuf.message.Message):
    """MsgUnjailResponse defines the Msg/Unjail response type"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgUnjailResponse = MsgUnjailResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.gov.v1beta1.gov_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryProposalRequest(google.protobuf.message.Message):
    """QueryProposalRequest is the request type for the Query/Proposal RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["proposal_id",b"proposal_id"]) -> None: ...
global___QueryProposalRequest = QueryProposalRequest

class QueryProposalResponse(google.protobuf.message.Message):
    """QueryProposalResponse is the response type for the Query/Proposal RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_FIELD_NUMBER: builtins.int
    @property
    def proposal(self) -> cosmos.gov.v1beta1.gov_pb2.Proposal: ...
    def __init__(self,
        *,
        proposal: typing.Optional[cosmos.gov.v1beta1.gov_pb2.Proposal] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proposal",b"proposal"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["proposal",b"proposal"]) -> None: ...
global___QueryProposalResponse = QueryProposalResponse

class QueryProposalsRequest(google.protobuf.message.Message):
    """QueryProposalsRequest is the request type for the Query/Proposals RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_STATUS_FIELD_NUMBER: builtins.int
    VOTER_FIELD_NUMBER: builtins.int
    DEPOSITOR_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    proposal_status: cosmos.gov.v1beta1.gov_pb2.ProposalStatus.ValueType
    """proposal_status defines the status of the proposals."""

    voter: typing.Text
    """voter defines the voter address for the proposals."""

    depositor: typing.Text
    """depositor defines the deposit addresses from the proposals."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        proposal_status: cosmos.gov.v1beta1.gov_pb2.ProposalStatus.ValueType = ...,
        voter: typing.Text = ...,
        depositor: typing.Text = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["depositor",b"depositor","pagination",b"pagination","proposal_status",b"proposal_status","voter",b"voter"]) -> None: ...
global___QueryProposalsRequest = QueryProposalsRequest

class QueryProposalsResponse(google.protobuf.message.Message):
    """QueryProposalsResponse is the response type for the Query/Proposals RPC
    method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSALS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def proposals(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.gov.v1beta1.gov_pb2.Proposal]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        proposals: typing.Optional[typing.Iterable[cosmos.gov.v1beta1.gov_pb2.Proposal]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","proposals",b"proposals"]) -> None: ...
global___QueryProposalsResponse = QueryProposalsResponse

class QueryVoteRequest(google.protobuf.message.Message):
    """QueryVoteRequest is the request type for the Query/Vote RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    VOTER_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    voter: typing.Text
    """voter defines the oter address for the proposals."""

    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        voter: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["proposal_id",b"proposal_id","voter",b"voter"]) -> None: ...
global___QueryVoteRequest = QueryVoteRequest

class QueryVoteResponse(google.protobuf.message.Message):
    """QueryVoteResponse is the response type for the Query/Vote RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTE_FIELD_NUMBER: builtins.int
    @property
    def vote(self) -> cosmos.gov.v1beta1.gov_pb2.Vote:
        """vote defined the queried vote."""
        pass
    def __init__(self,
        *,
        vote: typing.Optional[cosmos.gov.v1beta1.gov_pb2.Vote] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["vote",b"vote"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["vote",b"vote"]) -> None: ...
global___QueryVoteResponse = QueryVoteResponse

class QueryVotesRequest(google.protobuf.message.Message):
    """QueryVotesRequest is the request type for the Query/Votes RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","proposal_id",b"proposal_id"]) -> None: ...
global___QueryVotesRequest = QueryVotesRequest

class QueryVotesResponse(google.protobuf.message.Message):
    """QueryVotesResponse is the response type for the Query/Votes RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTES_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def votes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.gov.v1beta1.gov_pb2.Vote]:
        """votes defined the queried votes."""
        pass
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        votes: typing.Optional[typing.Iterable[cosmos.gov.v1beta1.gov_pb2.Vote]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","votes",b"votes"]) -> None: ...
global___QueryVotesResponse = QueryVotesResponse

class QueryParamsRequest(google.protobuf.message.Message):
    """QueryParamsRequest is the request type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_TYPE_FIELD_NUMBER: builtins.int
    params_type: typing.Text
    """params_type defines which parameters to query for, can be one of "voting",
    "tallying" or "deposit".
    """

    def __init__(self,
        *,
        params_type: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["params_type",b"params_type"]) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is the response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTING_PARAMS_FIELD_NUMBER: builtins.int
    DEPOSIT_PARAMS_FIELD_NUMBER: builtins.int
    TALLY_PARAMS_FIELD_NUMBER: builtins.int
    @property
    def voting_params(self) -> cosmos.gov.v1beta1.gov_pb2.VotingParams:
        """voting_params defines the parameters related to voting."""
        pass
    @property
    def deposit_params(self) -> cosmos.gov.v1beta1.gov_pb2.DepositParams:
        """deposit_params defines the parameters related to deposit."""
        pass
    @property
    def tally_params(self) -> cosmos.gov.v1beta1.gov_pb2.TallyParams:
        """tally_params defines the parameters related to tally."""
        pass
    def __init__(self,
        *,
        voting_params: typing.Optional[cosmos.gov.v1beta1.gov_pb2.VotingParams] = ...,
        deposit_params: typing.Optional[cosmos.gov.v1beta1.gov_pb2.DepositParams] = ...,
        tally_params: typing.Optional[cosmos.gov.v1beta1.gov_pb2.TallyParams] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["deposit_params",b"deposit_params","tally_params",b"tally_params","voting_params",b"voting_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["deposit_params",b"deposit_params","tally_params",b"tally_params","voting_params",b"voting_params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse

class QueryDepositRequest(google.protobuf.message.Message):
    """QueryDepositRequest is the request type for the Query/Deposit RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    DEPOSITOR_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    depositor: typing.Text
    """depositor defines the deposit addresses from the proposals."""

    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        depositor: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["depositor",b"depositor","proposal_id",b"proposal_id"]) -> None: ...
global___QueryDepositRequest = QueryDepositRequest

class QueryDepositResponse(google.protobuf.message.Message):
    """QueryDepositResponse is the response type for the Query/Deposit RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DEPOSIT_FIELD_NUMBER: builtins.int
    @property
    def deposit(self) -> cosmos.gov.v1beta1.gov_pb2.Deposit:
        """deposit defines the requested deposit."""
        pass
    def __init__(self,
        *,
        deposit: typing.Optional[cosmos.gov.v1beta1.gov_pb2.Deposit] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["deposit",b"deposit"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["deposit",b"deposit"]) -> None: ...
global___QueryDepositResponse = QueryDepositResponse

class QueryDepositsRequest(google.protobuf.message.Message):
    """QueryDepositsRequest is the request type for the Query/Deposits RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest:
        """pagination defines an optional pagination for the request."""
        pass
    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination","proposal_id",b"proposal_id"]) -> None: ...
global___QueryDepositsRequest = QueryDepositsRequest

class QueryDepositsResponse(google.protobuf.message.Message):
    """QueryDepositsResponse is the response type for the Query/Deposits RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DEPOSITS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def deposits(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.gov.v1beta1.gov_pb2.Deposit]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse:
        """pagination defines the pagination in the response."""
        pass
    def __init__(self,
        *,
        deposits: typing.Optional[typing.Iterable[cosmos.gov.v1beta1.gov_pb2.Deposit]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["deposits",b"deposits","pagination",b"pagination"]) -> None: ...
global___QueryDepositsResponse = QueryDepositsResponse

class QueryTallyResultRequest(google.protobuf.message.Message):
    """QueryTallyResultRequest is the request type for the Query/Tally RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    """proposal_id defines the unique id of the proposal."""

    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["proposal_id",b"proposal_id"]) -> None: ...
global___QueryTallyResultRequest = QueryTallyResultRequest

class QueryTallyResultResponse(google.protobuf.message.Message):
    """QueryTallyResultResponse is the response type for the Query/Tally RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TALLY_FIELD_NUMBER: builtins.int
    @property
    def tally(self) -> cosmos.gov.v1beta1.gov_pb2.TallyResult:
        """tally defines the requested tally."""
        pass
    def __init__(self,
        *,
        tally: typing.Optional[cosmos.gov.v1beta1.gov_pb2.TallyResult] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["tally",b"tally"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["tally",b"tally"]) -> None: ...
global___QueryTallyResultResponse = QueryTallyResultResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.vesting.v1beta1.tx_pb2
import grpc

class MsgStub:
    """Msg defines the bank Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    CreateVestingAccount: grpc.UnaryUnaryMultiCallable[
        cosmos.vesting.v1beta1.tx_pb2.MsgCreateVestingAccount,
        cosmos.vesting.v1beta1.tx_pb2.MsgCreateVestingAccountResponse]
    """CreateVestingAccount defines a method that enables creating a vesting
    account.
    """


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the bank Msg service."""
    @abc.abstractmethod
    def CreateVestingAccount(self,
        request: cosmos.vesting.v1beta1.tx_pb2.MsgCreateVestingAccount,
        context: grpc.ServicerContext,
    ) -> cosmos.vesting.v1beta1.tx_pb2.MsgCreateVestingAccountResponse:
        """CreateVestingAccount defines a method that enables creating a vesting
        account.
        """
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



# Copyright 2022 Nibiru Labs
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import asyncio
import logging

from nibiru import Sdk, Side

async def main() -> None:
    trader = Sdk.authorize("guard cream sadness conduct invite crumble clock pudding hole grit liar hotel maid produce squeeze return argue turtle know drive eight casino maze host")
    res = await trader.tx.perp.open_position(
        sender = trader.address,
        token_pair = "unusd:unibi",
        side=Side.BUY,
        quote_asset_amount="5",
        leverage="5",
        base_asset_amount_limit="5",
    )
    print(res)

if __name__ == "__main__":
    logging.basicConfig(level=logging.INFO)
    asyncio.get_event_loop().run_until_complete(main())



from nibiru.proto.perp.v1 import (
    tx_pb2 as tx,
    state_pb2 as state_pb,
)

from nibiru.proto.cosmos.base.v1beta1 import coin_pb2 as coin_pb
from nibiru.common import Side

class Perp:
    @staticmethod
    def remove_margin(sender: str, token_pair: str, margin: coin_pb.Coin):
        return tx.MsgRemoveMargin(
            sender = sender,
            token_pair = token_pair,
            margin = margin,
        )

    @staticmethod
    def add_margin(sender: str, token_pair: str, margin: coin_pb.Coin):
        return tx.MsgAddMargin(
            sender = sender,
            token_pair = token_pair,
            margin = margin,
        )

    @staticmethod
    def liquidate(sender: str, token_pair: str, trader: str):
        return tx.MsgLiquidate(
            sender = sender,
            token_pair = token_pair,
            trader = trader,
        )

    @staticmethod
    def open_position(sender: str, token_pair: str, side: Side, quote_asset_amount: str, leverage: str, base_asset_amount_limit: str):
        pb_side = state_pb.Side.BUY if side == Side.BUY else state_pb.SELL

        return tx.MsgOpenPosition(
            sender = sender,
            token_pair = token_pair,
            side = pb_side,
            quote_asset_amount = quote_asset_amount,
            leverage = leverage,
            base_asset_amount_limit = base_asset_amount_limit,
        )

    @staticmethod
    def close_position(sender: str, token_pair: str):
        return tx.MsgClosePosition(
            sender = sender,
            token_pair = token_pair,
        )



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Proof(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOTAL_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    LEAF_HASH_FIELD_NUMBER: builtins.int
    AUNTS_FIELD_NUMBER: builtins.int
    total: builtins.int
    index: builtins.int
    leaf_hash: builtins.bytes
    @property
    def aunts(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self,
        *,
        total: builtins.int = ...,
        index: builtins.int = ...,
        leaf_hash: builtins.bytes = ...,
        aunts: typing.Optional[typing.Iterable[builtins.bytes]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["aunts",b"aunts","index",b"index","leaf_hash",b"leaf_hash","total",b"total"]) -> None: ...
global___Proof = Proof

class ValueOp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    PROOF_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    """Encoded in ProofOp.Key."""

    @property
    def proof(self) -> global___Proof:
        """To encode in ProofOp.Data"""
        pass
    def __init__(self,
        *,
        key: builtins.bytes = ...,
        proof: typing.Optional[global___Proof] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proof",b"proof"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key","proof",b"proof"]) -> None: ...
global___ValueOp = ValueOp

class DominoOp(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    INPUT_FIELD_NUMBER: builtins.int
    OUTPUT_FIELD_NUMBER: builtins.int
    key: typing.Text
    input: typing.Text
    output: typing.Text
    def __init__(self,
        *,
        key: typing.Text = ...,
        input: typing.Text = ...,
        output: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["input",b"input","key",b"key","output",b"output"]) -> None: ...
global___DominoOp = DominoOp

class ProofOp(google.protobuf.message.Message):
    """ProofOp defines an operation used for calculating Merkle root
    The data could be arbitrary format, providing nessecary data
    for example neighbouring node hash
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    type: typing.Text
    key: builtins.bytes
    data: builtins.bytes
    def __init__(self,
        *,
        type: typing.Text = ...,
        key: builtins.bytes = ...,
        data: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data",b"data","key",b"key","type",b"type"]) -> None: ...
global___ProofOp = ProofOp

class ProofOps(google.protobuf.message.Message):
    """ProofOps is Merkle proof defined by the list of ProofOps"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OPS_FIELD_NUMBER: builtins.int
    @property
    def ops(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ProofOp]: ...
    def __init__(self,
        *,
        ops: typing.Optional[typing.Iterable[global___ProofOp]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["ops",b"ops"]) -> None: ...
global___ProofOps = ProofOps



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: stablecoin/params.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x17stablecoin/params.proto\x12\x14nibiru.stablecoin.v1\x1a\x14gogoproto/gogo.proto\"\x89\x02\n\x06Params\x12\x11\n\tcollRatio\x18\x01 \x01(\x03\x12\x10\n\x08\x66\x65\x65Ratio\x18\x02 \x01(\x03\x12\x12\n\nefFeeRatio\x18\x03 \x01(\x03\x12\x19\n\x11\x62onus_rate_recoll\x18\x04 \x01(\x03\x12\x41\n\x16\x64istr_epoch_identifier\x18\x05 \x01(\tB!\xf2\xde\x1f\x1dyaml:\"distr_epoch_identifier\"\x12\x16\n\x0e\x61\x64justmentStep\x18\x06 \x01(\x03\x12\x17\n\x0fpriceLowerBound\x18\x07 \x01(\x03\x12\x17\n\x0fpriceUpperBound\x18\x08 \x01(\x03\x12\x1e\n\x16isCollateralRatioValid\x18\t \x01(\x08\x42\x32Z0github.com/NibiruChain/nibiru/x/stablecoin/typesb\x06proto3')



_PARAMS = DESCRIPTOR.message_types_by_name['Params']
Params = _reflection.GeneratedProtocolMessageType('Params', (_message.Message,), {
  'DESCRIPTOR' : _PARAMS,
  '__module__' : 'stablecoin.params_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.stablecoin.v1.Params)
  })
_sym_db.RegisterMessage(Params)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z0github.com/NibiruChain/nibiru/x/stablecoin/types'
  _PARAMS.fields_by_name['distr_epoch_identifier']._options = None
  _PARAMS.fields_by_name['distr_epoch_identifier']._serialized_options = b'\362\336\037\035yaml:\"distr_epoch_identifier\"'
  _PARAMS._serialized_start=72
  _PARAMS._serialized_end=337
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Capability(google.protobuf.message.Message):
    """Capability defines an implementation of an object capability. The index
    provided to a Capability must be globally unique.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INDEX_FIELD_NUMBER: builtins.int
    index: builtins.int
    def __init__(self,
        *,
        index: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["index",b"index"]) -> None: ...
global___Capability = Capability

class Owner(google.protobuf.message.Message):
    """Owner defines a single capability owner. An owner is defined by the name of
    capability and the module name.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MODULE_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    module: typing.Text
    name: typing.Text
    def __init__(self,
        *,
        module: typing.Text = ...,
        name: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["module",b"module","name",b"name"]) -> None: ...
global___Owner = Owner

class CapabilityOwners(google.protobuf.message.Message):
    """CapabilityOwners defines a set of owners of a single Capability. The set of
    owners must be unique.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OWNERS_FIELD_NUMBER: builtins.int
    @property
    def owners(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Owner]: ...
    def __init__(self,
        *,
        owners: typing.Optional[typing.Iterable[global___Owner]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["owners",b"owners"]) -> None: ...
global___CapabilityOwners = CapabilityOwners



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.auth.v1beta1.auth_pb2
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class BaseVestingAccount(google.protobuf.message.Message):
    """BaseVestingAccount implements the VestingAccount interface. It contains all
    the necessary fields needed for any vesting account implementation.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_ACCOUNT_FIELD_NUMBER: builtins.int
    ORIGINAL_VESTING_FIELD_NUMBER: builtins.int
    DELEGATED_FREE_FIELD_NUMBER: builtins.int
    DELEGATED_VESTING_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    @property
    def base_account(self) -> cosmos.auth.v1beta1.auth_pb2.BaseAccount: ...
    @property
    def original_vesting(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def delegated_free(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def delegated_vesting(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    end_time: builtins.int
    def __init__(self,
        *,
        base_account: typing.Optional[cosmos.auth.v1beta1.auth_pb2.BaseAccount] = ...,
        original_vesting: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        delegated_free: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        delegated_vesting: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        end_time: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_account",b"base_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_account",b"base_account","delegated_free",b"delegated_free","delegated_vesting",b"delegated_vesting","end_time",b"end_time","original_vesting",b"original_vesting"]) -> None: ...
global___BaseVestingAccount = BaseVestingAccount

class ContinuousVestingAccount(google.protobuf.message.Message):
    """ContinuousVestingAccount implements the VestingAccount interface. It
    continuously vests by unlocking coins linearly with respect to time.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_VESTING_ACCOUNT_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    @property
    def base_vesting_account(self) -> global___BaseVestingAccount: ...
    start_time: builtins.int
    def __init__(self,
        *,
        base_vesting_account: typing.Optional[global___BaseVestingAccount] = ...,
        start_time: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account","start_time",b"start_time"]) -> None: ...
global___ContinuousVestingAccount = ContinuousVestingAccount

class DelayedVestingAccount(google.protobuf.message.Message):
    """DelayedVestingAccount implements the VestingAccount interface. It vests all
    coins after a specific time, but non prior. In other words, it keeps them
    locked until a specified time.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_VESTING_ACCOUNT_FIELD_NUMBER: builtins.int
    @property
    def base_vesting_account(self) -> global___BaseVestingAccount: ...
    def __init__(self,
        *,
        base_vesting_account: typing.Optional[global___BaseVestingAccount] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> None: ...
global___DelayedVestingAccount = DelayedVestingAccount

class Period(google.protobuf.message.Message):
    """Period defines a length of time and amount of coins that will vest."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    LENGTH_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    length: builtins.int
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        length: builtins.int = ...,
        amount: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","length",b"length"]) -> None: ...
global___Period = Period

class PeriodicVestingAccount(google.protobuf.message.Message):
    """PeriodicVestingAccount implements the VestingAccount interface. It
    periodically vests by unlocking coins during each specified period.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_VESTING_ACCOUNT_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    VESTING_PERIODS_FIELD_NUMBER: builtins.int
    @property
    def base_vesting_account(self) -> global___BaseVestingAccount: ...
    start_time: builtins.int
    @property
    def vesting_periods(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Period]: ...
    def __init__(self,
        *,
        base_vesting_account: typing.Optional[global___BaseVestingAccount] = ...,
        start_time: builtins.int = ...,
        vesting_periods: typing.Optional[typing.Iterable[global___Period]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account","start_time",b"start_time","vesting_periods",b"vesting_periods"]) -> None: ...
global___PeriodicVestingAccount = PeriodicVestingAccount

class PermanentLockedAccount(google.protobuf.message.Message):
    """PermanentLockedAccount implements the VestingAccount interface. It does
    not ever release coins, locking them indefinitely. Coins in this account can
    still be used for delegating and for governance votes even while locked.

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_VESTING_ACCOUNT_FIELD_NUMBER: builtins.int
    @property
    def base_vesting_account(self) -> global___BaseVestingAccount: ...
    def __init__(self,
        *,
        base_vesting_account: typing.Optional[global___BaseVestingAccount] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_vesting_account",b"base_vesting_account"]) -> None: ...
global___PermanentLockedAccount = PermanentLockedAccount



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import pricefeed.tx_pb2

class MsgStub:
    """Msg defines the pricefeed Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    PostPrice: grpc.UnaryUnaryMultiCallable[
        pricefeed.tx_pb2.MsgPostPrice,
        pricefeed.tx_pb2.MsgPostPriceResponse]
    """PostPrice defines a method for creating a new post price"""


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the pricefeed Msg service."""
    @abc.abstractmethod
    def PostPrice(self,
        request: pricefeed.tx_pb2.MsgPostPrice,
        context: grpc.ServicerContext,
    ) -> pricefeed.tx_pb2.MsgPostPriceResponse:
        """PostPrice defines a method for creating a new post price"""
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



from grpc import Channel
from typing import List

from nibiru.proto.cosmos.base.v1beta1 import coin_pb2 as cosmos_base_coin_pb
from nibiru.proto.dex.v1 import (
    query_pb2_grpc as dex_query,
    query_pb2 as dex_type,
)
from nibiru.proto.cosmos.base.query.v1beta1.pagination_pb2 import PageRequest

class Dex:
    """
    Dex allows to query the endpoints made available by the Nibiru Chain's DEX module.
    """
    def __init__(
        self,
        channel: Channel
    ):
        self.api = dex_query.QueryStub(channel)

    async def params(self):
        req = dex_type.QueryParamsRequest()
        return await self.api.Params(req)

    async def pool_number(self):
        req = dex_type.QueryPoolsRequest()
        return await self.api.PoolNumber(req)

    async def pool(self, pool_id: int):
        req = dex_type.QueryPoolRequest(poolId = pool_id)
        return await self.api.Pool(req)

    async def pools(self, **kwargs):
        '''
        Returns all available pools
        Parameters:
            key (bytes): The page key for the next page. Only key or offset should be set
            offset (int): The number of entries to skip. Only offset or key should be set
            limit (int): The number of max results in the page
            count_total (bool): Indicates if the response should contain the total number of results
            reverse (bool): Indicates if the results should be returned in descending order
        '''
        req = dex_type.QueryPoolsRequest(pagination = PageRequest(
                key = kwargs.get("key"),
                offset = kwargs.get("offset"),
                limit = kwargs.get("limit"),
                count_total = kwargs.get("count_total"),
                reverse = kwargs.get("reverse"),
            ),
        )
        return await self.api.Pools(req)

    async def pool_params(self, pool_id: int):
        req = dex_type.QueryPoolParamsRequest(poolId = pool_id)
        return await self.api.PoolParams(req)

    async def num_pools(self):
        req = dex_type.QueryNumPoolsRequest()
        return await self.api.NumPools(req)

    async def total_liquidity(self):
        req = dex_type.QueryTotalLiquidityRequest()
        return await self.api.TotalLiquidity(req)

    async def total_pool_liquidity(self, pool_id: int):
        req = dex_type.QueryTotalPoolLiquidityRequest(poolId = pool_id)
        return await self.api.TotalPoolLiquidity(req)

    async def total_shares(self, pool_id: int):
        req = dex_type.QueryTotalSharesRequest(poolId = pool_id)
        return await self.api.TotalShares(req)

    async def spot_price(self, pool_id: int, token_in_denom: str, token_out_denom: str):
        req = dex_type.QuerySpotPriceRequest(
            poolId = pool_id,
            tokenInDenom = token_in_denom,
            tokenOutDenom = token_out_denom,
        )
        return await self.api.SpotPrice(req)

    async def estimate_swap_exact_amount_in(self, pool_id: int, token_in: cosmos_base_coin_pb.Coin, token_out_denom: str):
        req = dex_type.QuerySwapExactAmountInRequest(
            poolId = pool_id,
            tokenIn = token_in,
            tokenOutDenom = token_out_denom,
        )
        return await self.api.EstimateSwapExactAmountIn(req)

    async def estimate_swap_exact_amount_out(self, pool_id: int, token_out: cosmos_base_coin_pb.Coin, token_in_denom: str):
        req = dex_type.QuerySwapExactAmountOutRequest(
            poolId = pool_id,
            tokenOut = token_out,
            tokenInDenom = token_in_denom,
        )
        return await self.api.EstimateSwapExactAmountOut(req)

    async def estimate_join_exact_amount_in(self, pool_id: int, tokens_in: List[cosmos_base_coin_pb.Coin]):
        req = dex_type.QueryJoinExactAmountInRequest(
            poolId = pool_id,
            tokensIn = tokens_in,
        )
        return await self.api.EstimateJoinExactAmountIn(req)

    async def estimate_join_exact_amount_out(self, pool_id: int):
        req = dex_type.QueryJoinExactAmountOutRequest(
            poolId = pool_id,
        )
        return await self.api.EstimateJoinExactAmountOut(req)

    async def estimate_exit_exact_amount_in(self, pool_id: int, num_shares: int):
        req = dex_type.QueryExitExactAmountInRequest(
            poolId = pool_id,
            poolSharesIn = num_shares
        )
        return await self.api.EstimateExitExactAmountIn(req)

    async def estimate_exit_exact_amount_out(self, pool_id: int):
        req = dex_type.QueryExitExactAmountOutRequest(
            poolId = pool_id,
        )
        return await self.api.EstimateExitExactAmountOut(req)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.tx.v1beta1.service_pb2
import grpc

class ServiceStub:
    """Service defines a gRPC service for interacting with transactions."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Simulate: grpc.UnaryUnaryMultiCallable[
        cosmos.tx.v1beta1.service_pb2.SimulateRequest,
        cosmos.tx.v1beta1.service_pb2.SimulateResponse]
    """Simulate simulates executing a transaction for estimating gas usage."""

    GetTx: grpc.UnaryUnaryMultiCallable[
        cosmos.tx.v1beta1.service_pb2.GetTxRequest,
        cosmos.tx.v1beta1.service_pb2.GetTxResponse]
    """GetTx fetches a tx by hash."""

    BroadcastTx: grpc.UnaryUnaryMultiCallable[
        cosmos.tx.v1beta1.service_pb2.BroadcastTxRequest,
        cosmos.tx.v1beta1.service_pb2.BroadcastTxResponse]
    """BroadcastTx broadcast transaction."""

    GetTxsEvent: grpc.UnaryUnaryMultiCallable[
        cosmos.tx.v1beta1.service_pb2.GetTxsEventRequest,
        cosmos.tx.v1beta1.service_pb2.GetTxsEventResponse]
    """GetTxsEvent fetches txs by event."""

    GetBlockWithTxs: grpc.UnaryUnaryMultiCallable[
        cosmos.tx.v1beta1.service_pb2.GetBlockWithTxsRequest,
        cosmos.tx.v1beta1.service_pb2.GetBlockWithTxsResponse]
    """GetBlockWithTxs fetches a block with decoded txs.

    Since: cosmos-sdk 0.45.2
    """


class ServiceServicer(metaclass=abc.ABCMeta):
    """Service defines a gRPC service for interacting with transactions."""
    @abc.abstractmethod
    def Simulate(self,
        request: cosmos.tx.v1beta1.service_pb2.SimulateRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.tx.v1beta1.service_pb2.SimulateResponse:
        """Simulate simulates executing a transaction for estimating gas usage."""
        pass

    @abc.abstractmethod
    def GetTx(self,
        request: cosmos.tx.v1beta1.service_pb2.GetTxRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.tx.v1beta1.service_pb2.GetTxResponse:
        """GetTx fetches a tx by hash."""
        pass

    @abc.abstractmethod
    def BroadcastTx(self,
        request: cosmos.tx.v1beta1.service_pb2.BroadcastTxRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.tx.v1beta1.service_pb2.BroadcastTxResponse:
        """BroadcastTx broadcast transaction."""
        pass

    @abc.abstractmethod
    def GetTxsEvent(self,
        request: cosmos.tx.v1beta1.service_pb2.GetTxsEventRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.tx.v1beta1.service_pb2.GetTxsEventResponse:
        """GetTxsEvent fetches txs by event."""
        pass

    @abc.abstractmethod
    def GetBlockWithTxs(self,
        request: cosmos.tx.v1beta1.service_pb2.GetBlockWithTxsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.tx.v1beta1.service_pb2.GetBlockWithTxsResponse:
        """GetBlockWithTxs fetches a block with decoded txs.

        Since: cosmos-sdk 0.45.2
        """
        pass


def add_ServiceServicer_to_server(servicer: ServiceServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.slashing.v1beta1.slashing_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class GenesisState(google.protobuf.message.Message):
    """GenesisState defines the slashing module's genesis state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    SIGNING_INFOS_FIELD_NUMBER: builtins.int
    MISSED_BLOCKS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> cosmos.slashing.v1beta1.slashing_pb2.Params:
        """params defines all the paramaters of related to deposit."""
        pass
    @property
    def signing_infos(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___SigningInfo]:
        """signing_infos represents a map between validator addresses and their
        signing infos.
        """
        pass
    @property
    def missed_blocks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorMissedBlocks]:
        """missed_blocks represents a map between validator addresses and their
        missed blocks.
        """
        pass
    def __init__(self,
        *,
        params: typing.Optional[cosmos.slashing.v1beta1.slashing_pb2.Params] = ...,
        signing_infos: typing.Optional[typing.Iterable[global___SigningInfo]] = ...,
        missed_blocks: typing.Optional[typing.Iterable[global___ValidatorMissedBlocks]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["missed_blocks",b"missed_blocks","params",b"params","signing_infos",b"signing_infos"]) -> None: ...
global___GenesisState = GenesisState

class SigningInfo(google.protobuf.message.Message):
    """SigningInfo stores validator signing info of corresponding address."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    VALIDATOR_SIGNING_INFO_FIELD_NUMBER: builtins.int
    address: typing.Text
    """address is the validator address."""

    @property
    def validator_signing_info(self) -> cosmos.slashing.v1beta1.slashing_pb2.ValidatorSigningInfo:
        """validator_signing_info represents the signing info of this validator."""
        pass
    def __init__(self,
        *,
        address: typing.Text = ...,
        validator_signing_info: typing.Optional[cosmos.slashing.v1beta1.slashing_pb2.ValidatorSigningInfo] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validator_signing_info",b"validator_signing_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","validator_signing_info",b"validator_signing_info"]) -> None: ...
global___SigningInfo = SigningInfo

class ValidatorMissedBlocks(google.protobuf.message.Message):
    """ValidatorMissedBlocks contains array of missed blocks of corresponding
    address.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    MISSED_BLOCKS_FIELD_NUMBER: builtins.int
    address: typing.Text
    """address is the validator address."""

    @property
    def missed_blocks(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MissedBlock]:
        """missed_blocks is an array of missed blocks by the validator."""
        pass
    def __init__(self,
        *,
        address: typing.Text = ...,
        missed_blocks: typing.Optional[typing.Iterable[global___MissedBlock]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","missed_blocks",b"missed_blocks"]) -> None: ...
global___ValidatorMissedBlocks = ValidatorMissedBlocks

class MissedBlock(google.protobuf.message.Message):
    """MissedBlock contains height and missed status as boolean."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INDEX_FIELD_NUMBER: builtins.int
    MISSED_FIELD_NUMBER: builtins.int
    index: builtins.int
    """index is the height at which the block was missed."""

    missed: builtins.bool
    """missed is the missed status."""

    def __init__(self,
        *,
        index: builtins.int = ...,
        missed: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["index",b"index","missed",b"missed"]) -> None: ...
global___MissedBlock = MissedBlock



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import tendermint.abci.types_pb2

class ABCIApplicationStub:
    """----------------------------------------
    Service Definition

    """
    def __init__(self, channel: grpc.Channel) -> None: ...
    Echo: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestEcho,
        tendermint.abci.types_pb2.ResponseEcho]

    Flush: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestFlush,
        tendermint.abci.types_pb2.ResponseFlush]

    Info: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestInfo,
        tendermint.abci.types_pb2.ResponseInfo]

    SetOption: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestSetOption,
        tendermint.abci.types_pb2.ResponseSetOption]

    DeliverTx: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestDeliverTx,
        tendermint.abci.types_pb2.ResponseDeliverTx]

    CheckTx: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestCheckTx,
        tendermint.abci.types_pb2.ResponseCheckTx]

    Query: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestQuery,
        tendermint.abci.types_pb2.ResponseQuery]

    Commit: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestCommit,
        tendermint.abci.types_pb2.ResponseCommit]

    InitChain: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestInitChain,
        tendermint.abci.types_pb2.ResponseInitChain]

    BeginBlock: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestBeginBlock,
        tendermint.abci.types_pb2.ResponseBeginBlock]

    EndBlock: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestEndBlock,
        tendermint.abci.types_pb2.ResponseEndBlock]

    ListSnapshots: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestListSnapshots,
        tendermint.abci.types_pb2.ResponseListSnapshots]

    OfferSnapshot: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestOfferSnapshot,
        tendermint.abci.types_pb2.ResponseOfferSnapshot]

    LoadSnapshotChunk: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestLoadSnapshotChunk,
        tendermint.abci.types_pb2.ResponseLoadSnapshotChunk]

    ApplySnapshotChunk: grpc.UnaryUnaryMultiCallable[
        tendermint.abci.types_pb2.RequestApplySnapshotChunk,
        tendermint.abci.types_pb2.ResponseApplySnapshotChunk]


class ABCIApplicationServicer(metaclass=abc.ABCMeta):
    """----------------------------------------
    Service Definition

    """
    @abc.abstractmethod
    def Echo(self,
        request: tendermint.abci.types_pb2.RequestEcho,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseEcho: ...

    @abc.abstractmethod
    def Flush(self,
        request: tendermint.abci.types_pb2.RequestFlush,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseFlush: ...

    @abc.abstractmethod
    def Info(self,
        request: tendermint.abci.types_pb2.RequestInfo,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseInfo: ...

    @abc.abstractmethod
    def SetOption(self,
        request: tendermint.abci.types_pb2.RequestSetOption,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseSetOption: ...

    @abc.abstractmethod
    def DeliverTx(self,
        request: tendermint.abci.types_pb2.RequestDeliverTx,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseDeliverTx: ...

    @abc.abstractmethod
    def CheckTx(self,
        request: tendermint.abci.types_pb2.RequestCheckTx,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseCheckTx: ...

    @abc.abstractmethod
    def Query(self,
        request: tendermint.abci.types_pb2.RequestQuery,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseQuery: ...

    @abc.abstractmethod
    def Commit(self,
        request: tendermint.abci.types_pb2.RequestCommit,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseCommit: ...

    @abc.abstractmethod
    def InitChain(self,
        request: tendermint.abci.types_pb2.RequestInitChain,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseInitChain: ...

    @abc.abstractmethod
    def BeginBlock(self,
        request: tendermint.abci.types_pb2.RequestBeginBlock,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseBeginBlock: ...

    @abc.abstractmethod
    def EndBlock(self,
        request: tendermint.abci.types_pb2.RequestEndBlock,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseEndBlock: ...

    @abc.abstractmethod
    def ListSnapshots(self,
        request: tendermint.abci.types_pb2.RequestListSnapshots,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseListSnapshots: ...

    @abc.abstractmethod
    def OfferSnapshot(self,
        request: tendermint.abci.types_pb2.RequestOfferSnapshot,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseOfferSnapshot: ...

    @abc.abstractmethod
    def LoadSnapshotChunk(self,
        request: tendermint.abci.types_pb2.RequestLoadSnapshotChunk,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseLoadSnapshotChunk: ...

    @abc.abstractmethod
    def ApplySnapshotChunk(self,
        request: tendermint.abci.types_pb2.RequestApplySnapshotChunk,
        context: grpc.ServicerContext,
    ) -> tendermint.abci.types_pb2.ResponseApplySnapshotChunk: ...


def add_ABCIApplicationServicer_to_server(servicer: ABCIApplicationServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.message
import perp.v1.state_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgRemoveMargin(google.protobuf.message.Message):
    """-------------------------- RemoveMargin --------------------------

    MsgRemoveMargin: Msg to remove margin.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    MARGIN_FIELD_NUMBER: builtins.int
    sender: typing.Text
    token_pair: typing.Text
    @property
    def margin(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        sender: typing.Text = ...,
        token_pair: typing.Text = ...,
        margin: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["margin",b"margin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["margin",b"margin","sender",b"sender","token_pair",b"token_pair"]) -> None: ...
global___MsgRemoveMargin = MsgRemoveMargin

class MsgRemoveMarginResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MARGIN_OUT_FIELD_NUMBER: builtins.int
    FUNDING_PAYMENT_FIELD_NUMBER: builtins.int
    @property
    def margin_out(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """MarginOut: tokens transferred back to the trader"""
        pass
    funding_payment: typing.Text
    def __init__(self,
        *,
        margin_out: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        funding_payment: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["margin_out",b"margin_out"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["funding_payment",b"funding_payment","margin_out",b"margin_out"]) -> None: ...
global___MsgRemoveMarginResponse = MsgRemoveMarginResponse

class MsgAddMargin(google.protobuf.message.Message):
    """-------------------------- AddMargin --------------------------

    MsgAddMargin: Msg to remove margin.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    MARGIN_FIELD_NUMBER: builtins.int
    sender: typing.Text
    token_pair: typing.Text
    @property
    def margin(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        sender: typing.Text = ...,
        token_pair: typing.Text = ...,
        margin: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["margin",b"margin"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["margin",b"margin","sender",b"sender","token_pair",b"token_pair"]) -> None: ...
global___MsgAddMargin = MsgAddMargin

class MsgAddMarginResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FUNDING_PAYMENT_FIELD_NUMBER: builtins.int
    POSITION_FIELD_NUMBER: builtins.int
    funding_payment: typing.Text
    @property
    def position(self) -> perp.v1.state_pb2.Position: ...
    def __init__(self,
        *,
        funding_payment: typing.Text = ...,
        position: typing.Optional[perp.v1.state_pb2.Position] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["position",b"position"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["funding_payment",b"funding_payment","position",b"position"]) -> None: ...
global___MsgAddMarginResponse = MsgAddMarginResponse

class MsgLiquidate(google.protobuf.message.Message):
    """-------------------------- Liquidate --------------------------

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    TRADER_FIELD_NUMBER: builtins.int
    sender: typing.Text
    """Sender is the liquidator address"""

    token_pair: typing.Text
    """TokenPair is the identifier for the position's virtual pool"""

    trader: typing.Text
    """Trader is the address of the owner of the position"""

    def __init__(self,
        *,
        sender: typing.Text = ...,
        token_pair: typing.Text = ...,
        trader: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sender",b"sender","token_pair",b"token_pair","trader",b"trader"]) -> None: ...
global___MsgLiquidate = MsgLiquidate

class MsgLiquidateResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FEE_TO_LIQUIDATOR_FIELD_NUMBER: builtins.int
    FEE_TO_PERP_ECOSYSTEM_FUND_FIELD_NUMBER: builtins.int
    @property
    def fee_to_liquidator(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    @property
    def fee_to_perp_ecosystem_fund(self) -> cosmos.base.v1beta1.coin_pb2.Coin: ...
    def __init__(self,
        *,
        fee_to_liquidator: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        fee_to_perp_ecosystem_fund: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["fee_to_liquidator",b"fee_to_liquidator","fee_to_perp_ecosystem_fund",b"fee_to_perp_ecosystem_fund"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fee_to_liquidator",b"fee_to_liquidator","fee_to_perp_ecosystem_fund",b"fee_to_perp_ecosystem_fund"]) -> None: ...
global___MsgLiquidateResponse = MsgLiquidateResponse

class MsgOpenPosition(google.protobuf.message.Message):
    """-------------------------- OpenPosition --------------------------

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    SIDE_FIELD_NUMBER: builtins.int
    QUOTE_ASSET_AMOUNT_FIELD_NUMBER: builtins.int
    LEVERAGE_FIELD_NUMBER: builtins.int
    BASE_ASSET_AMOUNT_LIMIT_FIELD_NUMBER: builtins.int
    sender: typing.Text
    token_pair: typing.Text
    side: perp.v1.state_pb2.Side.ValueType
    quote_asset_amount: typing.Text
    leverage: typing.Text
    base_asset_amount_limit: typing.Text
    def __init__(self,
        *,
        sender: typing.Text = ...,
        token_pair: typing.Text = ...,
        side: perp.v1.state_pb2.Side.ValueType = ...,
        quote_asset_amount: typing.Text = ...,
        leverage: typing.Text = ...,
        base_asset_amount_limit: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_asset_amount_limit",b"base_asset_amount_limit","leverage",b"leverage","quote_asset_amount",b"quote_asset_amount","sender",b"sender","side",b"side","token_pair",b"token_pair"]) -> None: ...
global___MsgOpenPosition = MsgOpenPosition

class MsgOpenPositionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgOpenPositionResponse = MsgOpenPositionResponse

class MsgClosePosition(google.protobuf.message.Message):
    """-------------------------- ClosePosition --------------------------

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    sender: typing.Text
    token_pair: typing.Text
    def __init__(self,
        *,
        sender: typing.Text = ...,
        token_pair: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["sender",b"sender","token_pair",b"token_pair"]) -> None: ...
global___MsgClosePosition = MsgClosePosition

class MsgClosePositionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgClosePositionResponse = MsgClosePositionResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Minter(google.protobuf.message.Message):
    """Minter represents the minting state."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INFLATION_FIELD_NUMBER: builtins.int
    ANNUAL_PROVISIONS_FIELD_NUMBER: builtins.int
    inflation: typing.Text
    """current annual inflation rate"""

    annual_provisions: typing.Text
    """current annual expected provisions"""

    def __init__(self,
        *,
        inflation: typing.Text = ...,
        annual_provisions: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["annual_provisions",b"annual_provisions","inflation",b"inflation"]) -> None: ...
global___Minter = Minter

class Params(google.protobuf.message.Message):
    """Params holds parameters for the mint module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MINT_DENOM_FIELD_NUMBER: builtins.int
    INFLATION_RATE_CHANGE_FIELD_NUMBER: builtins.int
    INFLATION_MAX_FIELD_NUMBER: builtins.int
    INFLATION_MIN_FIELD_NUMBER: builtins.int
    GOAL_BONDED_FIELD_NUMBER: builtins.int
    BLOCKS_PER_YEAR_FIELD_NUMBER: builtins.int
    mint_denom: typing.Text
    """type of coin to mint"""

    inflation_rate_change: typing.Text
    """maximum annual change in inflation rate"""

    inflation_max: typing.Text
    """maximum inflation rate"""

    inflation_min: typing.Text
    """minimum inflation rate"""

    goal_bonded: typing.Text
    """goal of percent bonded atoms"""

    blocks_per_year: builtins.int
    """expected blocks per year"""

    def __init__(self,
        *,
        mint_denom: typing.Text = ...,
        inflation_rate_change: typing.Text = ...,
        inflation_max: typing.Text = ...,
        inflation_min: typing.Text = ...,
        goal_bonded: typing.Text = ...,
        blocks_per_year: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["blocks_per_year",b"blocks_per_year","goal_bonded",b"goal_bonded","inflation_max",b"inflation_max","inflation_min",b"inflation_min","inflation_rate_change",b"inflation_rate_change","mint_denom",b"mint_denom"]) -> None: ...
global___Params = Params



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import stablecoin.query_pb2

class QueryStub:
    """Query defines the gRPC querier service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Params: grpc.UnaryUnaryMultiCallable[
        stablecoin.query_pb2.QueryParamsRequest,
        stablecoin.query_pb2.QueryParamsResponse]
    """Parameters queries the parameters of the x/stablecoin module."""

    ModuleAccountBalances: grpc.UnaryUnaryMultiCallable[
        stablecoin.query_pb2.QueryModuleAccountBalances,
        stablecoin.query_pb2.QueryModuleAccountBalancesResponse]
    """ModuleAccountBalances queries the account balance of x/stablecoin."""

    CirculatingSupplies: grpc.UnaryUnaryMultiCallable[
        stablecoin.query_pb2.QueryCirculatingSupplies,
        stablecoin.query_pb2.QueryCirculatingSuppliesResponse]

    LiquidityRatioInfo: grpc.UnaryUnaryMultiCallable[
        stablecoin.query_pb2.QueryLiquidityRatioInfoRequest,
        stablecoin.query_pb2.QueryLiquidityRatioInfoResponse]


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service."""
    @abc.abstractmethod
    def Params(self,
        request: stablecoin.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> stablecoin.query_pb2.QueryParamsResponse:
        """Parameters queries the parameters of the x/stablecoin module."""
        pass

    @abc.abstractmethod
    def ModuleAccountBalances(self,
        request: stablecoin.query_pb2.QueryModuleAccountBalances,
        context: grpc.ServicerContext,
    ) -> stablecoin.query_pb2.QueryModuleAccountBalancesResponse:
        """ModuleAccountBalances queries the account balance of x/stablecoin."""
        pass

    @abc.abstractmethod
    def CirculatingSupplies(self,
        request: stablecoin.query_pb2.QueryCirculatingSupplies,
        context: grpc.ServicerContext,
    ) -> stablecoin.query_pb2.QueryCirculatingSuppliesResponse: ...

    @abc.abstractmethod
    def LiquidityRatioInfo(self,
        request: stablecoin.query_pb2.QueryLiquidityRatioInfoRequest,
        context: grpc.ServicerContext,
    ) -> stablecoin.query_pb2.QueryLiquidityRatioInfoResponse: ...


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgGrantAllowance(google.protobuf.message.Message):
    """MsgGrantAllowance adds permission for Grantee to spend up to Allowance
    of fees from the account of Granter.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    GRANTEE_FIELD_NUMBER: builtins.int
    ALLOWANCE_FIELD_NUMBER: builtins.int
    granter: typing.Text
    """granter is the address of the user granting an allowance of their funds."""

    grantee: typing.Text
    """grantee is the address of the user being granted an allowance of another user's funds."""

    @property
    def allowance(self) -> google.protobuf.any_pb2.Any:
        """allowance can be any of basic and filtered fee allowance."""
        pass
    def __init__(self,
        *,
        granter: typing.Text = ...,
        grantee: typing.Text = ...,
        allowance: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["allowance",b"allowance"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allowance",b"allowance","grantee",b"grantee","granter",b"granter"]) -> None: ...
global___MsgGrantAllowance = MsgGrantAllowance

class MsgGrantAllowanceResponse(google.protobuf.message.Message):
    """MsgGrantAllowanceResponse defines the Msg/GrantAllowanceResponse response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgGrantAllowanceResponse = MsgGrantAllowanceResponse

class MsgRevokeAllowance(google.protobuf.message.Message):
    """MsgRevokeAllowance removes any existing Allowance from Granter to Grantee."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    GRANTER_FIELD_NUMBER: builtins.int
    GRANTEE_FIELD_NUMBER: builtins.int
    granter: typing.Text
    """granter is the address of the user granting an allowance of their funds."""

    grantee: typing.Text
    """grantee is the address of the user being granted an allowance of another user's funds."""

    def __init__(self,
        *,
        granter: typing.Text = ...,
        grantee: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["grantee",b"grantee","granter",b"granter"]) -> None: ...
global___MsgRevokeAllowance = MsgRevokeAllowance

class MsgRevokeAllowanceResponse(google.protobuf.message.Message):
    """MsgRevokeAllowanceResponse defines the Msg/RevokeAllowanceResponse response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgRevokeAllowanceResponse = MsgRevokeAllowanceResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class BaseAccount(google.protobuf.message.Message):
    """BaseAccount defines a base account type. It contains all the necessary fields
    for basic account functionality. Any custom account type should extend this
    type for additional functionality (e.g. vesting).
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    PUB_KEY_FIELD_NUMBER: builtins.int
    ACCOUNT_NUMBER_FIELD_NUMBER: builtins.int
    SEQUENCE_FIELD_NUMBER: builtins.int
    address: typing.Text
    @property
    def pub_key(self) -> google.protobuf.any_pb2.Any: ...
    account_number: builtins.int
    sequence: builtins.int
    def __init__(self,
        *,
        address: typing.Text = ...,
        pub_key: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        account_number: builtins.int = ...,
        sequence: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pub_key",b"pub_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["account_number",b"account_number","address",b"address","pub_key",b"pub_key","sequence",b"sequence"]) -> None: ...
global___BaseAccount = BaseAccount

class ModuleAccount(google.protobuf.message.Message):
    """ModuleAccount defines an account for modules that holds coins on a pool."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BASE_ACCOUNT_FIELD_NUMBER: builtins.int
    NAME_FIELD_NUMBER: builtins.int
    PERMISSIONS_FIELD_NUMBER: builtins.int
    @property
    def base_account(self) -> global___BaseAccount: ...
    name: typing.Text
    @property
    def permissions(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
    def __init__(self,
        *,
        base_account: typing.Optional[global___BaseAccount] = ...,
        name: typing.Text = ...,
        permissions: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["base_account",b"base_account"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["base_account",b"base_account","name",b"name","permissions",b"permissions"]) -> None: ...
global___ModuleAccount = ModuleAccount

class Params(google.protobuf.message.Message):
    """Params defines the parameters for the auth module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MAX_MEMO_CHARACTERS_FIELD_NUMBER: builtins.int
    TX_SIG_LIMIT_FIELD_NUMBER: builtins.int
    TX_SIZE_COST_PER_BYTE_FIELD_NUMBER: builtins.int
    SIG_VERIFY_COST_ED25519_FIELD_NUMBER: builtins.int
    SIG_VERIFY_COST_SECP256K1_FIELD_NUMBER: builtins.int
    max_memo_characters: builtins.int
    tx_sig_limit: builtins.int
    tx_size_cost_per_byte: builtins.int
    sig_verify_cost_ed25519: builtins.int
    sig_verify_cost_secp256k1: builtins.int
    def __init__(self,
        *,
        max_memo_characters: builtins.int = ...,
        tx_sig_limit: builtins.int = ...,
        tx_size_cost_per_byte: builtins.int = ...,
        sig_verify_cost_ed25519: builtins.int = ...,
        sig_verify_cost_secp256k1: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_memo_characters",b"max_memo_characters","sig_verify_cost_ed25519",b"sig_verify_cost_ed25519","sig_verify_cost_secp256k1",b"sig_verify_cost_secp256k1","tx_sig_limit",b"tx_sig_limit","tx_size_cost_per_byte",b"tx_size_cost_per_byte"]) -> None: ...
global___Params = Params



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Params(google.protobuf.message.Message):
    """Params defines the parameters for the module."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    COLLRATIO_FIELD_NUMBER: builtins.int
    FEERATIO_FIELD_NUMBER: builtins.int
    EFFEERATIO_FIELD_NUMBER: builtins.int
    BONUS_RATE_RECOLL_FIELD_NUMBER: builtins.int
    DISTR_EPOCH_IDENTIFIER_FIELD_NUMBER: builtins.int
    ADJUSTMENTSTEP_FIELD_NUMBER: builtins.int
    PRICELOWERBOUND_FIELD_NUMBER: builtins.int
    PRICEUPPERBOUND_FIELD_NUMBER: builtins.int
    ISCOLLATERALRATIOVALID_FIELD_NUMBER: builtins.int
    collRatio: builtins.int
    """collRatio is the ratio needed as collateral to exchange for stables"""

    feeRatio: builtins.int
    """feeRatio is the ratio taken as fees when minting or burning stables"""

    efFeeRatio: builtins.int
    """efFeeRatio is the ratio taken from the fees that goes to Ecosystem Fund"""

    bonus_rate_recoll: builtins.int
    """BonusRateRecoll is the percentage of extra stablecoin value given to the caller
    of 'Recollateralize' in units of governance tokens.
    """

    distr_epoch_identifier: typing.Text
    """distr_epoch_identifier defines the frequnecy of update for the collateral ratio"""

    adjustmentStep: builtins.int
    """adjustmentStep is the size of the step taken when updating the collateral ratio"""

    priceLowerBound: builtins.int
    """priceLowerBound is the lower bound for the stable coin to trigger a collateral ratio update"""

    priceUpperBound: builtins.int
    """priceUpperBound is the upper bound for the stable coin to trigger a collateral ratio update"""

    isCollateralRatioValid: builtins.bool
    """isCollateralRatioValid checks if the collateral ratio is correctly updated"""

    def __init__(self,
        *,
        collRatio: builtins.int = ...,
        feeRatio: builtins.int = ...,
        efFeeRatio: builtins.int = ...,
        bonus_rate_recoll: builtins.int = ...,
        distr_epoch_identifier: typing.Text = ...,
        adjustmentStep: builtins.int = ...,
        priceLowerBound: builtins.int = ...,
        priceUpperBound: builtins.int = ...,
        isCollateralRatioValid: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["adjustmentStep",b"adjustmentStep","bonus_rate_recoll",b"bonus_rate_recoll","collRatio",b"collRatio","distr_epoch_identifier",b"distr_epoch_identifier","efFeeRatio",b"efFeeRatio","feeRatio",b"feeRatio","isCollateralRatioValid",b"isCollateralRatioValid","priceLowerBound",b"priceLowerBound","priceUpperBound",b"priceUpperBound"]) -> None: ...
global___Params = Params



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import grpc
import stablecoin.tx_pb2

class MsgStub:
    """Msg defines the x/stablecoin Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    MintStable: grpc.UnaryUnaryMultiCallable[
        stablecoin.tx_pb2.MsgMintStable,
        stablecoin.tx_pb2.MsgMintStableResponse]
    """MintStable defines a method for trading a mixture of GOV and COLL to mint an 
    equivalent value of stablecoins.
    """

    BurnStable: grpc.UnaryUnaryMultiCallable[
        stablecoin.tx_pb2.MsgBurnStable,
        stablecoin.tx_pb2.MsgBurnStableResponse]
    """BurnStable defines a method for redeeming/burning stablecoins to receive an 
    equivalent value as a mixture of governance and collateral tokens.
    """

    Recollateralize: grpc.UnaryUnaryMultiCallable[
        stablecoin.tx_pb2.MsgRecollateralize,
        stablecoin.tx_pb2.MsgRecollateralizeResponse]
    """Recollateralize defines a method for manually adding collateral to the 
    protocol in exchange for an equivalent stablecoin value in governance tokens 
    plus a small bonus.
    """

    Buyback: grpc.UnaryUnaryMultiCallable[
        stablecoin.tx_pb2.MsgBuyback,
        stablecoin.tx_pb2.MsgBuybackResponse]
    """Buyback defines a method for manually adding NIBI to the protocol 
    in exchange for an equivalent stablecoin value in collateral, effectively 
    executing a share buyback for Nibiru Chain. The NIBI purchased by the protocol 
    is then burned, distributing value to all NIBI hodlers.
    """


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the x/stablecoin Msg service."""
    @abc.abstractmethod
    def MintStable(self,
        request: stablecoin.tx_pb2.MsgMintStable,
        context: grpc.ServicerContext,
    ) -> stablecoin.tx_pb2.MsgMintStableResponse:
        """MintStable defines a method for trading a mixture of GOV and COLL to mint an 
        equivalent value of stablecoins.
        """
        pass

    @abc.abstractmethod
    def BurnStable(self,
        request: stablecoin.tx_pb2.MsgBurnStable,
        context: grpc.ServicerContext,
    ) -> stablecoin.tx_pb2.MsgBurnStableResponse:
        """BurnStable defines a method for redeeming/burning stablecoins to receive an 
        equivalent value as a mixture of governance and collateral tokens.
        """
        pass

    @abc.abstractmethod
    def Recollateralize(self,
        request: stablecoin.tx_pb2.MsgRecollateralize,
        context: grpc.ServicerContext,
    ) -> stablecoin.tx_pb2.MsgRecollateralizeResponse:
        """Recollateralize defines a method for manually adding collateral to the 
        protocol in exchange for an equivalent stablecoin value in governance tokens 
        plus a small bonus.
        """
        pass

    @abc.abstractmethod
    def Buyback(self,
        request: stablecoin.tx_pb2.MsgBuyback,
        context: grpc.ServicerContext,
    ) -> stablecoin.tx_pb2.MsgBuybackResponse:
        """Buyback defines a method for manually adding NIBI to the protocol 
        in exchange for an equivalent stablecoin value in collateral, effectively 
        executing a share buyback for Nibiru Chain. The NIBI purchased by the protocol 
        is then burned, distributing value to all NIBI hodlers.
        """
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



# Copyright 2021 Nibiru Labs
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Nibiru Exchange API client for Python. Example only."""

import asyncio
import logging

from nibiru.client import Client
from nibiru.network import Network

async def main() -> None:
    network = Network.local()
    client = Client(network, insecure=True)
    resp = await client.dex.params()
    print(resp)

if __name__ == '__main__':
    logging.basicConfig(level=logging.INFO)
    asyncio.get_event_loop().run_until_complete(main())



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import dex.v1.tx_pb2
import grpc

class MsgStub:
    """Msg defines the Msg service."""
    def __init__(self, channel: grpc.Channel) -> None: ...
    CreatePool: grpc.UnaryUnaryMultiCallable[
        dex.v1.tx_pb2.MsgCreatePool,
        dex.v1.tx_pb2.MsgCreatePoolResponse]
    """Used to create a pool."""

    JoinPool: grpc.UnaryUnaryMultiCallable[
        dex.v1.tx_pb2.MsgJoinPool,
        dex.v1.tx_pb2.MsgJoinPoolResponse]
    """Join a pool as a liquidity provider."""

    ExitPool: grpc.UnaryUnaryMultiCallable[
        dex.v1.tx_pb2.MsgExitPool,
        dex.v1.tx_pb2.MsgExitPoolResponse]
    """Exit a pool position by returning LP shares"""

    SwapAssets: grpc.UnaryUnaryMultiCallable[
        dex.v1.tx_pb2.MsgSwapAssets,
        dex.v1.tx_pb2.MsgSwapAssetsResponse]
    """Swap assets in a pool"""


class MsgServicer(metaclass=abc.ABCMeta):
    """Msg defines the Msg service."""
    @abc.abstractmethod
    def CreatePool(self,
        request: dex.v1.tx_pb2.MsgCreatePool,
        context: grpc.ServicerContext,
    ) -> dex.v1.tx_pb2.MsgCreatePoolResponse:
        """Used to create a pool."""
        pass

    @abc.abstractmethod
    def JoinPool(self,
        request: dex.v1.tx_pb2.MsgJoinPool,
        context: grpc.ServicerContext,
    ) -> dex.v1.tx_pb2.MsgJoinPoolResponse:
        """Join a pool as a liquidity provider."""
        pass

    @abc.abstractmethod
    def ExitPool(self,
        request: dex.v1.tx_pb2.MsgExitPool,
        context: grpc.ServicerContext,
    ) -> dex.v1.tx_pb2.MsgExitPoolResponse:
        """Exit a pool position by returning LP shares"""
        pass

    @abc.abstractmethod
    def SwapAssets(self,
        request: dex.v1.tx_pb2.MsgSwapAssets,
        context: grpc.ServicerContext,
    ) -> dex.v1.tx_pb2.MsgSwapAssetsResponse:
        """Swap assets in a pool"""
        pass


def add_MsgServicer_to_server(servicer: MsgServicer, server: grpc.Server) -> None: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgCreateVestingAccount(google.protobuf.message.Message):
    """MsgCreateVestingAccount defines a message that enables creating a vesting
    account.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    FROM_ADDRESS_FIELD_NUMBER: builtins.int
    TO_ADDRESS_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    END_TIME_FIELD_NUMBER: builtins.int
    DELAYED_FIELD_NUMBER: builtins.int
    from_address: typing.Text
    to_address: typing.Text
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    end_time: builtins.int
    delayed: builtins.bool
    def __init__(self,
        *,
        from_address: typing.Text = ...,
        to_address: typing.Text = ...,
        amount: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        end_time: builtins.int = ...,
        delayed: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","delayed",b"delayed","end_time",b"end_time","from_address",b"from_address","to_address",b"to_address"]) -> None: ...
global___MsgCreateVestingAccount = MsgCreateVestingAccount

class MsgCreateVestingAccountResponse(google.protobuf.message.Message):
    """MsgCreateVestingAccountResponse defines the Msg/CreateVestingAccount response type."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgCreateVestingAccountResponse = MsgCreateVestingAccountResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.descriptor_pb2
import google.protobuf.internal.extension_dict
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

GOPROTO_ENUM_PREFIX_FIELD_NUMBER: builtins.int
GOPROTO_ENUM_STRINGER_FIELD_NUMBER: builtins.int
ENUM_STRINGER_FIELD_NUMBER: builtins.int
ENUM_CUSTOMNAME_FIELD_NUMBER: builtins.int
ENUMDECL_FIELD_NUMBER: builtins.int
ENUMVALUE_CUSTOMNAME_FIELD_NUMBER: builtins.int
GOPROTO_GETTERS_ALL_FIELD_NUMBER: builtins.int
GOPROTO_ENUM_PREFIX_ALL_FIELD_NUMBER: builtins.int
GOPROTO_STRINGER_ALL_FIELD_NUMBER: builtins.int
VERBOSE_EQUAL_ALL_FIELD_NUMBER: builtins.int
FACE_ALL_FIELD_NUMBER: builtins.int
GOSTRING_ALL_FIELD_NUMBER: builtins.int
POPULATE_ALL_FIELD_NUMBER: builtins.int
STRINGER_ALL_FIELD_NUMBER: builtins.int
ONLYONE_ALL_FIELD_NUMBER: builtins.int
EQUAL_ALL_FIELD_NUMBER: builtins.int
DESCRIPTION_ALL_FIELD_NUMBER: builtins.int
TESTGEN_ALL_FIELD_NUMBER: builtins.int
BENCHGEN_ALL_FIELD_NUMBER: builtins.int
MARSHALER_ALL_FIELD_NUMBER: builtins.int
UNMARSHALER_ALL_FIELD_NUMBER: builtins.int
STABLE_MARSHALER_ALL_FIELD_NUMBER: builtins.int
SIZER_ALL_FIELD_NUMBER: builtins.int
GOPROTO_ENUM_STRINGER_ALL_FIELD_NUMBER: builtins.int
ENUM_STRINGER_ALL_FIELD_NUMBER: builtins.int
UNSAFE_MARSHALER_ALL_FIELD_NUMBER: builtins.int
UNSAFE_UNMARSHALER_ALL_FIELD_NUMBER: builtins.int
GOPROTO_EXTENSIONS_MAP_ALL_FIELD_NUMBER: builtins.int
GOPROTO_UNRECOGNIZED_ALL_FIELD_NUMBER: builtins.int
GOGOPROTO_IMPORT_FIELD_NUMBER: builtins.int
PROTOSIZER_ALL_FIELD_NUMBER: builtins.int
COMPARE_ALL_FIELD_NUMBER: builtins.int
TYPEDECL_ALL_FIELD_NUMBER: builtins.int
ENUMDECL_ALL_FIELD_NUMBER: builtins.int
GOPROTO_REGISTRATION_FIELD_NUMBER: builtins.int
MESSAGENAME_ALL_FIELD_NUMBER: builtins.int
GOPROTO_SIZECACHE_ALL_FIELD_NUMBER: builtins.int
GOPROTO_UNKEYED_ALL_FIELD_NUMBER: builtins.int
GOPROTO_GETTERS_FIELD_NUMBER: builtins.int
GOPROTO_STRINGER_FIELD_NUMBER: builtins.int
VERBOSE_EQUAL_FIELD_NUMBER: builtins.int
FACE_FIELD_NUMBER: builtins.int
GOSTRING_FIELD_NUMBER: builtins.int
POPULATE_FIELD_NUMBER: builtins.int
STRINGER_FIELD_NUMBER: builtins.int
ONLYONE_FIELD_NUMBER: builtins.int
EQUAL_FIELD_NUMBER: builtins.int
DESCRIPTION_FIELD_NUMBER: builtins.int
TESTGEN_FIELD_NUMBER: builtins.int
BENCHGEN_FIELD_NUMBER: builtins.int
MARSHALER_FIELD_NUMBER: builtins.int
UNMARSHALER_FIELD_NUMBER: builtins.int
STABLE_MARSHALER_FIELD_NUMBER: builtins.int
SIZER_FIELD_NUMBER: builtins.int
UNSAFE_MARSHALER_FIELD_NUMBER: builtins.int
UNSAFE_UNMARSHALER_FIELD_NUMBER: builtins.int
GOPROTO_EXTENSIONS_MAP_FIELD_NUMBER: builtins.int
GOPROTO_UNRECOGNIZED_FIELD_NUMBER: builtins.int
PROTOSIZER_FIELD_NUMBER: builtins.int
COMPARE_FIELD_NUMBER: builtins.int
TYPEDECL_FIELD_NUMBER: builtins.int
MESSAGENAME_FIELD_NUMBER: builtins.int
GOPROTO_SIZECACHE_FIELD_NUMBER: builtins.int
GOPROTO_UNKEYED_FIELD_NUMBER: builtins.int
NULLABLE_FIELD_NUMBER: builtins.int
EMBED_FIELD_NUMBER: builtins.int
CUSTOMTYPE_FIELD_NUMBER: builtins.int
CUSTOMNAME_FIELD_NUMBER: builtins.int
JSONTAG_FIELD_NUMBER: builtins.int
MORETAGS_FIELD_NUMBER: builtins.int
CASTTYPE_FIELD_NUMBER: builtins.int
CASTKEY_FIELD_NUMBER: builtins.int
CASTVALUE_FIELD_NUMBER: builtins.int
STDTIME_FIELD_NUMBER: builtins.int
STDDURATION_FIELD_NUMBER: builtins.int
WKTPOINTER_FIELD_NUMBER: builtins.int
CASTREPEATED_FIELD_NUMBER: builtins.int
goproto_enum_prefix: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, builtins.bool]

goproto_enum_stringer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, builtins.bool]

enum_stringer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, builtins.bool]

enum_customname: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, typing.Text]

enumdecl: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumOptions, builtins.bool]

enumvalue_customname: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.EnumValueOptions, typing.Text]

goproto_getters_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_enum_prefix_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_stringer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

verbose_equal_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

face_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

gostring_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

populate_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

stringer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

onlyone_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

equal_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

description_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

testgen_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

benchgen_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

marshaler_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

unmarshaler_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

stable_marshaler_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

sizer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_enum_stringer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

enum_stringer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

unsafe_marshaler_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

unsafe_unmarshaler_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_extensions_map_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_unrecognized_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

gogoproto_import: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

protosizer_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

compare_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

typedecl_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

enumdecl_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_registration: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

messagename_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_sizecache_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_unkeyed_all: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FileOptions, builtins.bool]

goproto_getters: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

goproto_stringer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

verbose_equal: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

face: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

gostring: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

populate: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

stringer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

onlyone: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

equal: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

description: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

testgen: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

benchgen: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

marshaler: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

unmarshaler: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

stable_marshaler: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

sizer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

unsafe_marshaler: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

unsafe_unmarshaler: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

goproto_extensions_map: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

goproto_unrecognized: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

protosizer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

compare: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

typedecl: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

messagename: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

goproto_sizecache: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

goproto_unkeyed: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.MessageOptions, builtins.bool]

nullable: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, builtins.bool]

embed: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, builtins.bool]

customtype: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

customname: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

jsontag: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

moretags: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

casttype: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

castkey: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

castvalue: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]

stdtime: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, builtins.bool]

stdduration: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, builtins.bool]

wktpointer: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, builtins.bool]

castrepeated: google.protobuf.internal.extension_dict._ExtensionFieldDescriptor[google.protobuf.descriptor_pb2.FieldOptions, typing.Text]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class Plan(google.protobuf.message.Message):
    """Plan specifies information about a planned upgrade and when it should occur."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    UPGRADED_CLIENT_STATE_FIELD_NUMBER: builtins.int
    name: typing.Text
    """Sets the name for the upgrade. This name will be used by the upgraded
    version of the software to apply any special "on-upgrade" commands during
    the first BeginBlock method after the upgrade is applied. It is also used
    to detect whether a software version can handle a given upgrade. If no
    upgrade handler with this name has been set in the software, it will be
    assumed that the software is out-of-date when the upgrade Time or Height is
    reached and the software will exit.
    """

    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """Deprecated: Time based upgrades have been deprecated. Time based upgrade logic
        has been removed from the SDK.
        If this field is not empty, an error will be thrown.
        """
        pass
    height: builtins.int
    """The height at which the upgrade must be performed.
    Only used if Time is not set.
    """

    info: typing.Text
    """Any application specific upgrade info to be included on-chain
    such as a git commit that validators could automatically upgrade to
    """

    @property
    def upgraded_client_state(self) -> google.protobuf.any_pb2.Any:
        """Deprecated: UpgradedClientState field has been deprecated. IBC upgrade logic has been
        moved to the IBC module in the sub module 02-client.
        If this field is not empty, an error will be thrown.
        """
        pass
    def __init__(self,
        *,
        name: typing.Text = ...,
        time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        height: builtins.int = ...,
        info: typing.Text = ...,
        upgraded_client_state: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time",b"time","upgraded_client_state",b"upgraded_client_state"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","info",b"info","name",b"name","time",b"time","upgraded_client_state",b"upgraded_client_state"]) -> None: ...
global___Plan = Plan

class SoftwareUpgradeProposal(google.protobuf.message.Message):
    """SoftwareUpgradeProposal is a gov Content type for initiating a software
    upgrade.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    PLAN_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    @property
    def plan(self) -> global___Plan: ...
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        plan: typing.Optional[global___Plan] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["plan",b"plan"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["description",b"description","plan",b"plan","title",b"title"]) -> None: ...
global___SoftwareUpgradeProposal = SoftwareUpgradeProposal

class CancelSoftwareUpgradeProposal(google.protobuf.message.Message):
    """CancelSoftwareUpgradeProposal is a gov Content type for cancelling a software
    upgrade.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description",b"description","title",b"title"]) -> None: ...
global___CancelSoftwareUpgradeProposal = CancelSoftwareUpgradeProposal

class ModuleVersion(google.protobuf.message.Message):
    """ModuleVersion specifies a module and its consensus version.

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    NAME_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    name: typing.Text
    """name of the app module"""

    version: builtins.int
    """consensus version of the app module"""

    def __init__(self,
        *,
        name: typing.Text = ...,
        version: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["name",b"name","version",b"version"]) -> None: ...
global___ModuleVersion = ModuleVersion



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.query.v1beta1.pagination_pb2
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class MsgCreateIncentivizationProgram(google.protobuf.message.Message):
    """MsgCreateIncentivizationProgram is the request needed for the CreateIncentivizationProgram RPC."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    LP_DENOM_FIELD_NUMBER: builtins.int
    MIN_LOCKUP_DURATION_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    EPOCHS_FIELD_NUMBER: builtins.int
    INITIAL_FUNDS_FIELD_NUMBER: builtins.int
    sender: typing.Text
    """sender defines the sender of the msg that needs to be signed."""

    lp_denom: typing.Text
    """lp_denom defines the liquidity pool denom that we want to incentivize."""

    @property
    def min_lockup_duration(self) -> google.protobuf.duration_pb2.Duration:
        """min_lockup_duration defines the minimum lockup duration
        required on the lp_denom to get the incentivization.
        """
        pass
    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """start_time identifies when the incentivization starts."""
        pass
    epochs: builtins.int
    """epochs defines the number of epochs the incentivization
    program will last.
    """

    @property
    def initial_funds(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """initial_funds defines the initial funds to bootstrap the incentivization program's escrow.
        This is optional.
        """
        pass
    def __init__(self,
        *,
        sender: typing.Text = ...,
        lp_denom: typing.Text = ...,
        min_lockup_duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        start_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        epochs: builtins.int = ...,
        initial_funds: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["min_lockup_duration",b"min_lockup_duration","start_time",b"start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["epochs",b"epochs","initial_funds",b"initial_funds","lp_denom",b"lp_denom","min_lockup_duration",b"min_lockup_duration","sender",b"sender","start_time",b"start_time"]) -> None: ...
global___MsgCreateIncentivizationProgram = MsgCreateIncentivizationProgram

class MsgCreateIncentivizationProgramResponse(google.protobuf.message.Message):
    """MsgCreateIncentivizationProgramResponse is the response returned by the CreateIncentivizationProgram RPC."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROGRAM_ID_FIELD_NUMBER: builtins.int
    program_id: builtins.int
    """program_id defines the incentivization program unique identifier."""

    def __init__(self,
        *,
        program_id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["program_id",b"program_id"]) -> None: ...
global___MsgCreateIncentivizationProgramResponse = MsgCreateIncentivizationProgramResponse

class MsgFundIncentivizationProgram(google.protobuf.message.Message):
    """MsgFundIncentivizationProgram is the request for the FundIncentivizationProgram RPC."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SENDER_FIELD_NUMBER: builtins.int
    ID_FIELD_NUMBER: builtins.int
    FUNDS_FIELD_NUMBER: builtins.int
    sender: typing.Text
    """sender is the sender of the requests"""

    id: builtins.int
    """id marks the incentivization program id."""

    @property
    def funds(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """funds is the amount of money sender deposits in the program funding escrow."""
        pass
    def __init__(self,
        *,
        sender: typing.Text = ...,
        id: builtins.int = ...,
        funds: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["funds",b"funds","id",b"id","sender",b"sender"]) -> None: ...
global___MsgFundIncentivizationProgram = MsgFundIncentivizationProgram

class MsgFundIncentivizationProgramResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___MsgFundIncentivizationProgramResponse = MsgFundIncentivizationProgramResponse

class IncentivizationProgram(google.protobuf.message.Message):
    """IncentivizationProgram defines how an incentivization program looks like."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    ESCROW_ADDRESS_FIELD_NUMBER: builtins.int
    REMAINING_EPOCHS_FIELD_NUMBER: builtins.int
    LP_DENOM_FIELD_NUMBER: builtins.int
    MIN_LOCKUP_DURATION_FIELD_NUMBER: builtins.int
    START_TIME_FIELD_NUMBER: builtins.int
    id: builtins.int
    """id defines the unique uint64 id of the program"""

    escrow_address: typing.Text
    """escrow_address defines the escrow module account address
    for the incentivization program.
    """

    remaining_epochs: builtins.int
    """remaining_epochs defines the number of epochs left before the incentivization
    program can be considered finalized.
    """

    lp_denom: typing.Text
    """lp_denom defines the liquidity pool denom that is being incentivized."""

    @property
    def min_lockup_duration(self) -> google.protobuf.duration_pb2.Duration:
        """min_lockup_duration defines the minimum lp_denom lockup period
        required to get the epochs' rewards.
        """
        pass
    @property
    def start_time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """start_time defines the incentivization program start time."""
        pass
    def __init__(self,
        *,
        id: builtins.int = ...,
        escrow_address: typing.Text = ...,
        remaining_epochs: builtins.int = ...,
        lp_denom: typing.Text = ...,
        min_lockup_duration: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        start_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["min_lockup_duration",b"min_lockup_duration","start_time",b"start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["escrow_address",b"escrow_address","id",b"id","lp_denom",b"lp_denom","min_lockup_duration",b"min_lockup_duration","remaining_epochs",b"remaining_epochs","start_time",b"start_time"]) -> None: ...
global___IncentivizationProgram = IncentivizationProgram

class GenesisState(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INCENTIVIZATION_PROGRAMS_FIELD_NUMBER: builtins.int
    @property
    def incentivization_programs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___IncentivizationProgram]: ...
    def __init__(self,
        *,
        incentivization_programs: typing.Optional[typing.Iterable[global___IncentivizationProgram]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["incentivization_programs",b"incentivization_programs"]) -> None: ...
global___GenesisState = GenesisState

class QueryIncentivizationProgramRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ID_FIELD_NUMBER: builtins.int
    id: builtins.int
    def __init__(self,
        *,
        id: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["id",b"id"]) -> None: ...
global___QueryIncentivizationProgramRequest = QueryIncentivizationProgramRequest

class QueryIncentivizationProgramResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INCENTIVIZATION_PROGRAM_FIELD_NUMBER: builtins.int
    @property
    def incentivization_program(self) -> global___IncentivizationProgram: ...
    def __init__(self,
        *,
        incentivization_program: typing.Optional[global___IncentivizationProgram] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["incentivization_program",b"incentivization_program"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["incentivization_program",b"incentivization_program"]) -> None: ...
global___QueryIncentivizationProgramResponse = QueryIncentivizationProgramResponse

class QueryIncentivizationProgramsRequest(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageRequest: ...
    def __init__(self,
        *,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageRequest] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> None: ...
global___QueryIncentivizationProgramsRequest = QueryIncentivizationProgramsRequest

class QueryIncentivizationProgramsResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INCENTIVIZATION_PROGRAMS_FIELD_NUMBER: builtins.int
    PAGINATION_FIELD_NUMBER: builtins.int
    @property
    def incentivization_programs(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___IncentivizationProgram]: ...
    @property
    def pagination(self) -> cosmos.base.query.v1beta1.pagination_pb2.PageResponse: ...
    def __init__(self,
        *,
        incentivization_programs: typing.Optional[typing.Iterable[global___IncentivizationProgram]] = ...,
        pagination: typing.Optional[cosmos.base.query.v1beta1.pagination_pb2.PageResponse] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pagination",b"pagination"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["incentivization_programs",b"incentivization_programs","pagination",b"pagination"]) -> None: ...
global___QueryIncentivizationProgramsResponse = QueryIncentivizationProgramsResponse



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class PubKey(google.protobuf.message.Message):
    """PubKey defines a secp256k1 public key
    Key is the compressed form of the pubkey. The first byte depends is a 0x02 byte
    if the y-coordinate is the lexicographically largest of the two associated with
    the x-coordinate. Otherwise the first byte is a 0x03.
    This prefix is followed with the x-coordinate.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    def __init__(self,
        *,
        key: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key"]) -> None: ...
global___PubKey = PubKey

class PrivKey(google.protobuf.message.Message):
    """PrivKey defines a secp256k1 private key."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    def __init__(self,
        *,
        key: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key"]) -> None: ...
global___PrivKey = PrivKey



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import abc
import cosmos.gov.v1beta1.query_pb2
import grpc

class QueryStub:
    """Query defines the gRPC querier service for gov module"""
    def __init__(self, channel: grpc.Channel) -> None: ...
    Proposal: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryProposalRequest,
        cosmos.gov.v1beta1.query_pb2.QueryProposalResponse]
    """Proposal queries proposal details based on ProposalID."""

    Proposals: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryProposalsRequest,
        cosmos.gov.v1beta1.query_pb2.QueryProposalsResponse]
    """Proposals queries all proposals based on given status."""

    Vote: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryVoteRequest,
        cosmos.gov.v1beta1.query_pb2.QueryVoteResponse]
    """Vote queries voted information based on proposalID, voterAddr."""

    Votes: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryVotesRequest,
        cosmos.gov.v1beta1.query_pb2.QueryVotesResponse]
    """Votes queries votes of a given proposal."""

    Params: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryParamsRequest,
        cosmos.gov.v1beta1.query_pb2.QueryParamsResponse]
    """Params queries all parameters of the gov module."""

    Deposit: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryDepositRequest,
        cosmos.gov.v1beta1.query_pb2.QueryDepositResponse]
    """Deposit queries single deposit information based proposalID, depositAddr."""

    Deposits: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryDepositsRequest,
        cosmos.gov.v1beta1.query_pb2.QueryDepositsResponse]
    """Deposits queries all deposits of a single proposal."""

    TallyResult: grpc.UnaryUnaryMultiCallable[
        cosmos.gov.v1beta1.query_pb2.QueryTallyResultRequest,
        cosmos.gov.v1beta1.query_pb2.QueryTallyResultResponse]
    """TallyResult queries the tally of a proposal vote."""


class QueryServicer(metaclass=abc.ABCMeta):
    """Query defines the gRPC querier service for gov module"""
    @abc.abstractmethod
    def Proposal(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryProposalRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryProposalResponse:
        """Proposal queries proposal details based on ProposalID."""
        pass

    @abc.abstractmethod
    def Proposals(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryProposalsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryProposalsResponse:
        """Proposals queries all proposals based on given status."""
        pass

    @abc.abstractmethod
    def Vote(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryVoteRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryVoteResponse:
        """Vote queries voted information based on proposalID, voterAddr."""
        pass

    @abc.abstractmethod
    def Votes(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryVotesRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryVotesResponse:
        """Votes queries votes of a given proposal."""
        pass

    @abc.abstractmethod
    def Params(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryParamsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryParamsResponse:
        """Params queries all parameters of the gov module."""
        pass

    @abc.abstractmethod
    def Deposit(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryDepositRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryDepositResponse:
        """Deposit queries single deposit information based proposalID, depositAddr."""
        pass

    @abc.abstractmethod
    def Deposits(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryDepositsRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryDepositsResponse:
        """Deposits queries all deposits of a single proposal."""
        pass

    @abc.abstractmethod
    def TallyResult(self,
        request: cosmos.gov.v1beta1.query_pb2.QueryTallyResultRequest,
        context: grpc.ServicerContext,
    ) -> cosmos.gov.v1beta1.query_pb2.QueryTallyResultResponse:
        """TallyResult queries the tally of a proposal vote."""
        pass


def add_QueryServicer_to_server(servicer: QueryServicer, server: grpc.Server) -> None: ...
